-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'File Path'
C.description = 'Loads a Path from a file.'
C.category = 'repeat_instant'
C.color = im.ImVec4(1, 1, 0, 0.75)
C.pinSchema = {
  {dir = 'in', type = 'flow', name = 'flow'},
  {dir = 'in', type = 'flow', name = 'reset', impulse = true},
  {dir = 'in', type = 'string', name = 'file', description = 'File of the race'},
  {dir = 'in', type = 'bool', name = 'reverse', description = 'If the path should be reversed, if possible.'},

  {dir = 'out', type = 'flow', name = 'flow'},
  {dir = 'out', type = 'table', name = 'pathData', tableType = 'pathData', description = 'Data from the path for other nodes to process.'},
  {dir = 'out', type = 'string', name = 'name', description = 'Name of the Race'},
  {dir = 'out', type = 'string', name = 'desc', description = 'Description of the Race'},
  {dir = 'out', type = 'bool', name = 'branching', hidden= true, description = 'If the path is branching.'},
  {dir = 'out', type = 'bool', name = 'closed', hidden= true, description = 'If the path is closed.'},
  {dir = 'out', type = 'number', name = 'laps', hidden= true,  description = 'Default number of laps.'},
  {dir = 'out', type = 'number', name = 'checkpointCount', hidden= true,  description = 'Number of checkpoints in total (does not work for branching)'}
}

C.tags = {'scenario'}

function C:init(mgr, ...)
  self:reset()
end

function C:_executionStarted()
  self:reset()
end

function C:postInit()
  self.pinInLocal.file.allowFiles = {
    {"Race Files",".race.json"},
  }
end

function C:reset()
  self.path = nil
end

function C:drawCustomProperties()
  if im.Button("Open Race Editor") then
    if editor_raceEditor then
      editor_raceEditor.show()
    end
  end
  if editor_raceEditor then
    local fn = editor_raceEditor.getCurrentFilename()
    if fn then
      im.Text("Currently open file in editor:")
      im.Text(fn)
      if im.Button("Hardcode to File Pin") then
        self:_setHardcodedDummyInputPin(self.pinInLocal.file, fn)
      end
    end
  end
end

function C:work(args)
  if self.pinIn.reset.value then
    self:reset()
  end
  if self.pinIn.flow.value then
    if self.path == nil then
      local file = self.pinIn.file.value
      if not file then
        log('E', logTag, 'no race file')
        return
      end
      local json = readJsonFile(self.mgr:getRelativeAbsolutePath({file, file..'.race.json'}))

      if not json then
        log('E', logTag, 'unable to find race file: ' .. tostring(file))
        return
      end
      local path = require('/lua/ge/extensions/gameplay/race/path')("New Path")
      path:onDeserialized(json)
      if self.pinIn.reverse.value then
        path:reverse()
      end
      path:autoConfig()

      self.path = path
      self.pinOut.pathData.value = path
      self.pinOut.desc.value = path.description
      self.pinOut.name.value = path.name
      self.pinOut.branching.value = path.config.branching
      self.pinOut.closed.value = path.config.closed
      self.pinOut.laps.value = path.defaultLaps
      self.pinOut.checkpointCount.value = #(path.pathnodes.sorted)
    end
    self.pinOut.flow.value = self.pinIn.flow.value
  end
end

return _flowgraph_createNode(C)
